#!/bin/bash -u
DEBUG='FALSE'
TRUSTEDSYSTEMS="127.0.0.1 LOCAL .fnal.gov"

###########################################################
# setup args in the right order for making getopt evaluation
# nice and easy.  You'll need to read the manpages for more info
args=$(getopt -o hd -- "$@")
eval set -- "$args"

###########################################################
usage() {
    echo ''           >&2
    echo "$0 <%post number>" >&2
    echo "   This script is really just an easy to call" >&2
    echo "   wrapper" >&2
    exit 1
}

###########################################################
add_service() {
    SERVICE=${1}
    cat > ${TMPFILE} <<EOF
match /files/etc/hosts.allow/*[process = '${SERVICE}']
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Finding if we've /etc/hosts.allow ${SERVICE}"
        cat ${TMPFILE}
    fi

    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi
    echo ${MATCHES} | grep -q '(no matches)'
    if [[ $? -eq 0 ]]; then
        echo "${SERVICE}: ${TRUSTEDSYSTEMS}" > /etc/hosts.allow.rpmconf
        cat /etc/hosts.allow >> /etc/hosts.allow.rpmconf
        mv /etc/hosts.allow.rpmconf /etc/hosts.allow
    else
        for ADDR in ${TRUSTEDSYSTEMS}; do
            cat > ${TMPFILE} <<EOF
match /files/etc/hosts.allow/*[process = '${SERVICE}'][client = '${ADDR}']
EOF
            if [[ "${DEBUG}" == 'TRUE' ]]; then
                echo "Finding if we've /etc/hosts.allow ${SERVICE}: ${ADDR}"
                cat ${TMPFILE}
            fi

            MATCHES=$(augtool <${TMPFILE})
            if [[ $? -ne 0 ]]; then
                exit 1
            fi
            echo ${MATCHES} | grep -q '(no matches)'
            if [[ $? -eq 0 ]]; then
                cat > ${TMPFILE} <<EOF
ins client before /files/etc/hosts.allow/*[process = '${SERVICE}']/client[1]
set /files/etc/hosts.allow/*[process = '${SERVICE}']/client[1] ${ADDR}
save
EOF
                CLEANUP=$(augtool <${TMPFILE})
                if [[ $? -ne 0 ]]; then
                    exit 1
                fi
            fi
        done
    fi
}

###########################################################
for arg in $@; do
    case $1 in
        -- )
            # end of getopt args, shift off the -- and get out of the loop
            shift
            break 2
           ;;
         -d )
            ###########################################################
            # enable debugging
            DEBUG='TRUE'
            shift
           ;;
         -h )
            ###########################################################
            # get help
            usage
           ;;

    esac
done

if [[ "${DEBUG}" == 'TRUE' ]]; then
    set -x
fi

###########################################################

CURRENT_INSTALLS_OF_THIS_PACKAGE=${1:-0}

which augtool >/dev/null 2>&1
if [[ $? -ne 0 ]]; then
    echo "Could not find augtool" >&2
    exit 1
fi

NOW=$(date +%x_%T)
TMPFILE=$(mktemp)

if [[ ${CURRENT_INSTALLS_OF_THIS_PACKAGE} -eq 0 ]]; then
    ###########################################################
    logger -p auth.info 'Removing fermilab-conf_tcp-wrappers'

    cat > ${TMPFILE} <<EOF
match /files/etc/hosts.deny/*[process = 'ALL'][client = 'ALL']
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Finding if we've /etc/hosts.deny ALL:ALL"
        cat ${TMPFILE}
    fi

    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi
    echo ${MATCHES} | grep -q '(no matches)'
    if [[ $? -ne 0 ]]; then
        cat > ${TMPFILE} <<EOF
rm /files/etc/hosts.deny/*[process = 'ALL'][client = 'ALL']
save
EOF
        if [[ "${DEBUG}" == 'TRUE' ]]; then
            echo "Cleaning up /etc/hosts.deny ALL:ALL"
            cat ${TMPFILE}
        fi

        CLEANUP=$(augtool <${TMPFILE})
        if [[ $? -ne 0 ]]; then
            exit 1
        fi
    fi
else
    ###########################################################
    logger -p auth.info 'Installing fermilab-conf_tcp-wrappers'

    cat > ${TMPFILE} <<EOF
match /files/etc/hosts.deny/*[process = 'ALL'][client = 'ALL']
EOF
    if [[ "${DEBUG}" == 'TRUE' ]]; then
        echo "Finding if we've /etc/hosts.deny ALL:ALL"
        cat ${TMPFILE}
    fi

    MATCHES=$(augtool <${TMPFILE})
    if [[ $? -ne 0 ]]; then
        exit 1
    fi
    echo ${MATCHES} | grep -q '(no matches)'
    if [[ $? -eq 0 ]]; then
        echo "ALL: ALL" > /etc/hosts.deny.rpmconf
        cat /etc/hosts.deny >> /etc/hosts.deny.rpmconf
        mv /etc/hosts.deny /etc/hosts.deny.rpmsave
        mv /etc/hosts.deny.rpmconf /etc/hosts.deny
    fi

    add_service 'ALL'
    add_service 'portmap'
    add_service 'rpc.mountd'

    grep -q '#sshd: ALL' /etc/hosts.allow
    if [[ $? -eq 0 ]]; then
        echo '#sshd: ALL' >> /etc/hosts.allow
    fi
fi

###########################################################
rm -f ${TMPFILE}
restorecon -F /etc/hosts.allow /etc/hosts.deny >/dev/null 2>&1
chmod 644 /etc/hosts.allow /etc/hosts.deny >/dev/null 2>&1

